# -*- coding: utf-8 -*-
"""
update mesh files

USAGE

    python -m thyrsis.database.update_mesh [database.sqlite]

    update mesh files
"""
from builtins import str
import sys
import os
from ..settings import Settings
from ..database import _execute_statements
from ..database import sqlite

def update_mesh(db_list, dir_name):
    for db in db_list:
        # fix_print_with_import
        print("adding columns for table mailles in", db)
        conn = sqlite.connect(os.path.join(dir_name, db))
        cur = conn.cursor()
        print(os.path.join(dir_name, db))
        _execute_statements(cur, 'update_mesh.sql',project_SRID=27572)

        # fix_print_with_import
        print("updating surfaces")
        cur.execute("UPDATE mailles SET surface = ST_Area(GEOMETRY)")

        mailles = cur.execute("SELECT OGC_FID, a, b, c, d FROM mailles ORDER BY OGC_FID").fetchall()
        nmailles = len(mailles)

        columns = ['infiltration',
                    'epaisseur_aquifere',
                    'epaisseur_zs',
                    'epaisseur_zns',
                    'potentiel_reference',
                    'potentiel',
                    'permeabilite_zns']

        rows = cur.execute("SELECT "+", ".join(columns)+" FROM noeuds ORDER BY OGC_FID").fetchall()
        if len(rows):
            for j, column in enumerate(columns):
                if rows[0][j]:
                    # fix_print_with_import
                    print("updating", column)
                    values = []
                    for [mid, a, b, c, d] in mailles:
                        if d:
                            values.append(((rows[a-1][j]+rows[b-1][j]+rows[c-1][j]+rows[d-1][j])/4., mid))
                        else:
                            values.append(((rows[a-1][j]+rows[b-1][j]+rows[c-1][j])/3., mid))

                    cur.executemany("UPDATE mailles SET "+column+" = ? WHERE OGC_FID = ? ", values)

            cur.execute("UPDATE mailles SET transmissivite_x = permeabilite_x*epaisseur_zs")
            cur.execute("UPDATE mailles SET transmissivite_y = permeabilite_y*epaisseur_zs")

        conn.commit()
        conn.close()


if __name__ == "__main__":

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    if len(sys.argv) == 2 and sys.argv[1][-7:] == '.sqlite':
        db_list = [sys.argv[1]]
        dir_name = "."
        # fix_print_with_import
        print("upgrading", sys.argv[1], "database")
    else:
        mesh_dir = str(Settings().value("General", "defaultMeshDir"))
        db_list = sorted(os.listdir(mesh_dir))
        dir_name = mesh_dir
        # fix_print_with_import
        print("upgrading", mesh_dir, "directory")

    update(db_list, dir_name)
