# -*- coding: utf-8 -*-
"""Creates all databases and computations 

USAGE:
    python -m thyrsis.simulation.compute_validation [-init -run -results -a] [template] [-template] [-code_name]

OPTIONS

   -init : create databases
   -run : run simulations (compute)
   -results : calculate results
   -a : all : raz, init, compute and results
   template: ne traite que le cas template
   -template: ne traite pas le cas template
   -code_name : nom du code de calcul, sinon code déterminé par Settings
"""
from builtins import str

import sip
assert(sip.getapi('QVariant') == 2)

import os
import sys
import shutil
import numpy
import re

from ..database import sqlite
from ..spatialitemeshdataprovider import SpatialiteMeshDataProvider
from .compute import compute_mpi
from ..log import logger
from ..settings import Settings
from ..utilities import Timer

if __name__ =="__main__":
    logger.enable_console(True)
    logger.set_level('debug')

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    references = {'DIF_2015_ERA':('R1', u'2017-10-30', u'Bq/L', 0.01, 'Transport ZS'),
                'PEM_2008_U':('PZ7', u'2154-01-02', u'µg/L', 525., 'Transport 4 colonnes ZNS + ZS'),
                'PEM_2014_TRANSITOIRE':('PZ1', u'2001-04-20', 'm', 231.0, 'Ecoulement transitoire ZS'),
                'PEM_2014_U2014':('PZ7', u'1970-01-01', u'µg/L', 215.1, 'Transport 1 colonne ZNS + ZS'),
                'PEM_2014_U2016':('PZ4', u'2008-01-01', u'µg/L', 1.02, 'Transport 108 colonnes ZNS + ZS'),
                'PEM_2014_Z5':('PZ11', u'2000-01-14', u'µg/L', 2.03, 'Transport 105 colonnes ZNS + ZS'),
                'SACLAY_2015_ORPHEE':('F29', u'2056-12-31', u'Bq/L', 5257.8, 'Transport 1 colonne ZNS + ZS'),
                'SACLAY_2015_TRANSITOIRE':('F44', u'2004-08-07', 'm', 115.6, 'Ecoulement transitoire 38 colonnes ZNS + ZS'),
                'SACLAY_2015_B390':('LIM', u'2010-02-01', u'Bq/L', 1.87, 'Ecoulement et transport transitoire 1 colonne ZNS + ZS'),
                'TER_2013_TRANSITOIRE':('FCP900', u'2004-02-24', 'm', 88.47, 'Ecoulement transitoire 41 colonnes ZNS + ZS'),
                'TER_2013_U':('FCP901P', u'1992-01-01', u'µg/L', 803.0, 'Transport 2 colonnes ZNS + ZS'),
                'TER_2013_U_INCERT':('FCP901P', u'2068-01-01', u'µg/L', 69.5, 'Transport 1 colonne ZNS + ZS, 5 calculs'),
                }

    settings = Settings()

    init = False
    run = False
    resu = False
    raz = False
    results = references
    results_raz = False

    for arg in sys.argv[1:]:
        if arg == '-init': init = True
        elif arg == '-run': run = True
        elif arg == '-results': resu = True
        elif arg == '-a':
            init = True
            run = True
            resu = True
            raz = True
        elif arg[0] == '-' and arg[1:] in list(references.keys()):
            del results[arg[1:]]
        elif arg in list(references.keys()):
            if not results_raz:
                results = {}
                results_raz = True
            results[arg] = references[arg]
        elif arg[1:] in ('metis', 'openfoam'):
            logger.notice("setting code to", arg[1:])
            settings.setValue("General", "codeHydro", arg[1:])
            settings.save()
        else:
            logger.error("argument not recognized", arg)
            exit(1)

    thyrsis_dir = os.environ['THYRSIS']
    data_dir = os.path.join(thyrsis_dir, 'data')

    code = settings.value("General", "codeHydro")
    test_dir = 'thyrsis_plugin_validation_'+code
    logger.notice("\nrunning in "+test_dir+"\n")

    # B390 openfoam pas encore au point
    if code == 'openfoam':
        for key in list(results.keys()):
            if 'B390' in key:
                del results[key]

    if raz:
        if os.path.exists(test_dir):
            shutil.rmtree(test_dir)
        os.makedirs(test_dir)

    timer = Timer()
    report = '\nRAPPORT DE TEST\n' 
    report = '\nTemplate;Reference;Calculated;Difference (%);CPU time;Commentaire\n' 
    for template in sorted(results.keys()):

        logger.notice("\n"+template)
        sitename = template.split('_')[0]

        tpltdir = os.path.join(test_dir,template)
        if init:
            if os.path.exists(tpltdir):
                shutil.rmtree(tpltdir)
            os.makedirs(tpltdir)
            tpltname = template.split('_')[2]
            if tpltname == 'TRANSITOIRE':
                u_esui = os.path.join(data_dir, sitename, 'u_infiltration_transitoire.insat')
                u_edse = os.path.join(data_dir, sitename, 'u_infiltration_transitoire.sat')
                if os.path.exists(u_esui):
                    if init: shutil.copy2(u_esui, tpltdir)
                elif os.path.exists(u_edse):
                    if init: shutil.copy2(u_edse, tpltdir)
                else:
                    logger.error("infiltration file missing")
                    continue

        dbname = os.path.join(tpltdir,template.lower()+'.sqlite')


    #    if sitename == 'SACLAY' and init:
    #        u_ezns = os.path.join(install_dir, 'sites', sitename, 'input', 'u_epaisseurZNS.sat')
    #        if os.path.exists(u_ezns):
    #            shutil.copy2(u_ezns, tpltdir)
    #        else:
    #            logger.error(u"fichier d'épaisseur ZNS absent")

        if init: os.system("python -m thyrsis.database "+template+" "+dbname)

        if run: compute_mpi(dbname, True, True)
        
        if resu:
            conn = sqlite.connect(dbname)
            cur = conn.cursor()
            xy = cur.execute("SELECT X(GEOMETRY), Y(GEOMETRY) FROM points_interet WHERE nom='%s'"%(results[template][0])).fetchone()
            notice = results[template][0]+" "+str(xy[0])+" "+str(xy[1])+" "+results[template][1]
            logger.notice(template)
            logger.notice(notice)
            #report += template+'\n'+notice+'\n'
            column = 'potentiel' if results[template][2] == 'm' else 'concentration'
            provider = SpatialiteMeshDataProvider('dbname=%s crs=epsg:27572 resultColumn=%s'%(dbname,column))
            provider.setUnits(results[template][2])
            date_idx, = [i for i,d in enumerate(provider.dates()) if d[:10] == results[template][1]]
            #print xy, date_idx, provider.dates()[date_idx]
            value = provider.valuesAt(xy)[date_idx]
            vref = results[template][3]
            d = 100*(value - vref)/vref
            cputime = timer.reset("")
            notice = u"%s;%.5f;%.5f;%.1f;%s;%s\n"%(template, vref, value, d, cputime.strip(), results[template][4])
            logger.notice(notice)
            report += notice
            conn.close()
        else:
            timer.reset("")

    logger.notice(report)
    with open(os.path.join(test_dir, 'report-'+code+'.csv'), 'a') as fil:
        fil.write(report)

