# coding=utf-8
"""
Crée des colonnes de forme anulaires

USAGE

    python -m thyrsis.create_columns database.sqlite xmin,ymin,xmax,ymax debut duree
    
    avec :
        xmin,ymin,xmax,ymax coordonnées du cadre où créer les colonnes
        debut = début du calcul sous la forme '01/02/1971'
        duree = durée du calcul sous la forme '39 year'
    
"""
from builtins import map
from builtins import str
from builtins import range

import math


def create_columns(cur, cadre, debut, duree):
    """ Create injections (unsaturated columns)

    :param cur: cursor on a sqlite connection
    :type cur: sqlite3.Cursor
    :param cadre: xmin,ymin,xmax,ymax
    :type cadre: list
    :param debut: begin date
    :type debut: date
    :param duree: duration
    :type duree: string
    """
    project_SRID = str(cur.execute("SELECT DISTINCT st_srid(GEOMETRY) FROM noeuds").fetchone()[0])
    cur.execute("""
        SELECT 
            OGC_FID,
            X(GEOMETRY), Y(GEOMETRY),
            surface
            FROM noeuds
            ORDER BY OGC_FID""")
    for [ogc_fid, x, y, surface] in cur.fetchall():
        if x >= cadre[0] and x <= cadre[2] and y >= cadre[1] and y<= cadre[3]:
            cur.execute("""INSERT INTO injections(geometry, debut, duree, nom) 
                VALUES(CastToMultiPolygon(GeomFromText('%s', %s)), '%s', '%s', '%s')"""%(disk(x, y, surface),
                project_SRID, datetime.strptime(debut, "%d/%m/%Y"), duree, 'N'+str(ogc_fid)))


def disk(x, y, surface):
    """return a disk with center (x,y), and surface surface
    
    :param x: x coordinate
    :type x: float
    :param y: y coordinate
    :type y: float
    :param surface: area
    :type surface: float
    """
    
    nb_seg = 32
    nb_pt = nb_seg + 1 # il faut fermer le contour
    d_theta = 2*math.pi/nb_seg
    R = math.sqrt(2*surface/nb_seg/math.sin(d_theta))

    exterior = ["%f %f"%(x+R*math.cos(t*d_theta), y+R*math.sin(t*d_theta)) for t in range(nb_pt)]

    return "POLYGON((%s))"%(','.join(exterior))

if __name__ == "__main__":

    import sys
    from ..database import sqlite
    from datetime import datetime

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    if len(sys.argv) < 4:
        # fix_print_with_import
        print("error: wrong number of arguments (try create_columns.py --help)")
        exit(1)

    conn = sqlite.connect(sys.argv[1])
    cur = conn.cursor()

#    debut = '01/02/1971'
#    duree = '39 year'
    cur.execute("""UPDATE simulations SET debut = '%s',
            type_injection = 'flux'"""%(datetime.strptime(sys.argv[3], "%d/%m/%Y")))
    cur.execute("UPDATE dates_resultats SET duree = '%s'"%(sys.argv[4]))

    create_columns(cur, list(map(int,sys.argv[2].split(','))), sys.argv[3], sys.argv[4])
    conn.commit()
    
