from builtins import object
import platform

def get_exclusive_lock(name):
    """ Lock 

    :param name: lock name
    :type name: string

    :return: lock file path
    :rtype: string
    """
    import fcntl
    fd = open("/tmp/{}.lock".format(name), "w")
    fcntl.lockf(fd.fileno(), fcntl.LOCK_EX)
    return fd

def unlock_exclusive_lock(fd):
    """ Unlock 

    :param fd: lock file path
    :type fd: string
    """
    import fcntl
    fcntl.lockf(fd.fileno(), fcntl.LOCK_UN)
    fd.close()

class ExclusiveLock(object):
    """ Exclusive Lock class"""
    def __init__(self, name):
        """Constructor
        
        :param name: lock name
        :type name: string"""
        self.__name = name
    def __enter__(self):
        """ Activate lock

        :return: lock tmp file
        :rtype: string
        """
        self.__fd = get_exclusive_lock(self.__name)
        return self.__fd
    def __exit__(self, exc_type, exc_value, traceback):
        """ Unlock, protected method
        """
        unlock_exclusive_lock(self.__fd)
